package com.xhkjedu.smarking.service.report.reportother;

import com.xhkjedu.smarking.mapper.exam.*;
import com.xhkjedu.smarking.mapper.report.reportother.MsrExamMapper;
import com.xhkjedu.smarking.mapper.stupaper.MsPaperStudentMapper;
import com.xhkjedu.smarking.model.exam.*;
import com.xhkjedu.smarking.model.paper.MsPaper;
import com.xhkjedu.smarking.model.report.reportother.MsrExam;
import com.xhkjedu.smarking.model.report.reportstu.MsrStudent;
import com.xhkjedu.smarking.model.report.reportsubject.MsrSubject;
import com.xhkjedu.smarking.model.stupaper.MsPaperStudent;
import com.xhkjedu.smarking.utils.MarkingUtil;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description 阅卷报告-总体分析表 服务实现类
 * @Author auto
 * @Date 2024-12-09
 */
@Service
public class MsrExamService {
    @Resource
    private MsrExamMapper msrExamMapper;
    @Resource
    private MsExamMapper examMapper;
    @Resource
    private MsSubjectMapper subjectMapper;
    @Resource
    private MsClassMapper classMapper;
    @Resource
    private MsClassStudentMapper classStudentMapper;
    @Resource
    private MsPaperStudentMapper paperStudentMapper;

    /**
     * @Description 生成报告
     * @Date 2024/12/9 15:43
     * @Author YWX
     * @Param [examid]
     * @Return void
     **/
    public void generateExamReport(Integer examid) {
        MsExam exam = examMapper.selectByPrimaryKey(examid);
        Integer exammode = exam.getExammode();
        List<MsPaper> subjects = subjectMapper.listReportSubject(examid, exammode);//考试报告科目列表
        if (N_Utils.isListEmpty(subjects)) return;
        List<MsClass> classes = classMapper.listClassByExamId(examid);//获取考试关联班级
        List<MsClassStudent> students = classStudentMapper.listStudentByExamId(examid);//获取考试关联学生
        List<Integer> mpids = subjects.stream().map(MsPaper::getMpid).collect(Collectors.toList());
        List<MsPaperStudent> paperStudents = paperStudentMapper.listPaperStudentByExamId(examid, mpids);//试学生成绩列表

        Integer classnum = N_Utils.obj2Int(classes.stream().map(MsClass::getClassid).distinct().count());
        Integer totalnum = N_Utils.obj2Int(students.stream().map(MsClassStudent::getStudentid).distinct().count());
        Integer stunum = N_Utils.obj2Int(paperStudents.stream().filter(s -> s.getSstate().equals(2)).map(MsPaperStudent::getStudentid).distinct().count());
        Integer missnum = totalnum - stunum;
        List<MsrStudent> schoolStudents = new ArrayList<>();//学校学生成绩
        List<MsrStudent> schoolSubjectStudents = new ArrayList<>();//学校科目学生成绩
        List<MsrStudent> classStudents = new ArrayList<>();//班级学生成绩
        List<MsrStudent> classSubjectStudents = new ArrayList<>();//班级科目学生成绩
        Map<String, Integer> schoolStuRank = new HashMap<>();//学校学生分数排名
        Map<String, Integer> schoolSubStuRank = new HashMap<>();//学校科目学生分数排名
        Map<String, Integer> classStuRank = new HashMap<>();//班级学生分数排名
        Map<String, Integer> classSubStuRank = new HashMap<>();//班级科目学生分数排名
        Integer stuzero = exam.getStuzero();
        Map<Integer, List<MsPaperStudent>> schoolCollect = paperStudents.stream().collect(Collectors.groupingBy(MsPaperStudent::getStudentid));
        for (Map.Entry<Integer, List<MsPaperStudent>> entry : schoolCollect.entrySet()) {
            Integer studentid = entry.getKey();
            MsrStudent cs = new MsrStudent();
            List<MsPaperStudent> list = entry.getValue();
            cs.setClassid(0);
            cs.setStudentid(studentid);
            cs.setStuscore(list.stream().mapToDouble(MsPaperStudent::getStuscore).sum());
            cs.setSstate(list.stream().max(Comparator.comparing(MsPaperStudent::getSstate)).get().getSstate());
            schoolStudents.add(cs);
        }
        List<MsrStudent> paperStudents2 = new ArrayList<>();
        Map<String, List<MsClassStudent>> sClassStudent = students.stream().collect(Collectors.groupingBy(s -> s.getSubjectid() + "_" + s.getStudentid()));
        for (MsPaperStudent paperStudent : paperStudents) {
            String subjectid = paperStudent.getSubjectid();
            Integer studentid = paperStudent.getStudentid();
            Double stuscore = paperStudent.getStuscore();
            String key = subjectid + "_" + studentid;
            for (MsClassStudent student : sClassStudent.get(key)) {
                MsrStudent cs = new MsrStudent();
                cs.setExamid(examid);
                cs.setClassid(student.getClassid());
                cs.setStudentid(student.getStudentid());
                cs.setSubjectid(subjectid);
                cs.setStuscore(stuscore);
                paperStudents2.add(cs);
            }
        }
        String subjectid;
        Map<Integer, List<MsrStudent>> cCollect = paperStudents2.stream().collect(Collectors.groupingBy(MsrStudent::getClassid));
        for (Map.Entry<Integer, List<MsrStudent>> entry : cCollect.entrySet()) {
            Integer classid = entry.getKey();
            subjectid = "zf";
            List<MsrStudent> studentList = entry.getValue();

            Map<Integer, List<MsrStudent>> sCollect = studentList.stream().collect(Collectors.groupingBy(MsrStudent::getStudentid));
            List<MsrStudent> rstudents = new ArrayList<>();
            for (Map.Entry<Integer, List<MsrStudent>> csEntry : sCollect.entrySet()) {
                MsrStudent cs = new MsrStudent();
                cs.setExamid(examid);
                cs.setClassid(classid);
                cs.setStudentid(csEntry.getKey());
                cs.setSubjectid(subjectid);
                cs.setStuscore(csEntry.getValue().stream().mapToDouble(MsrStudent::getStuscore).sum());
                rstudents.add(cs);
            }
            setRank(rstudents, classStuRank, 3);//班级学生分数排名
            classStudents.addAll(rstudents);
            Map<String, List<MsrStudent>> subCollect = studentList.stream().collect(Collectors.groupingBy(MsrStudent::getSubjectid));
            for (Map.Entry<String, List<MsrStudent>> ssEntry : subCollect.entrySet()) {
                rstudents = ssEntry.getValue();
                setRank(rstudents, classSubStuRank, 4);//班级科目学生分数排名
                classSubjectStudents.addAll(rstudents);
            }
        }

        List<MsrStudent> schoolStudents2;//参与统计学生列表
        if (stuzero.equals(0)) {//0分学生不参与统计
            schoolStudents2 = schoolStudents.stream().filter(s -> s.getStuscore() > 0).collect(Collectors.toList());
        } else {
            schoolStudents2 = schoolStudents;
        }
        DoubleSummaryStatistics schoolStatistics = schoolStudents2.stream().filter(s -> s.getStuscore() > 0).mapToDouble(MsrStudent::getStuscore).summaryStatistics();
        double schoolStatisticsMax = N_Utils.handleInfinity(schoolStatistics.getMax());
        //最低分均不包括0分
        double schoolStatisticsMin = N_Utils.handleInfinity(schoolStudents2.stream().filter(s -> s.getStuscore() > 0).mapToDouble(MsrStudent::getStuscore).summaryStatistics().getMin());
        double schoolStatisticsAverage = N_Utils.formatDouble(schoolStatistics.getAverage(), 2);

        //总体分析
        MsrExam msrExam = new MsrExam();
        msrExam.setExamid(examid);
        msrExam.setClassnum(classnum);
        msrExam.setTotalnum(totalnum);
        msrExam.setStunum(stunum);
        msrExam.setMissnum(missnum);
        msrExam.setSubjectnum(subjects.size());
        msrExam.setMaxscore(schoolStatisticsMax);
        msrExam.setMinscore(schoolStatisticsMin);
        msrExam.setCreatetime(N_Utils.getSecondTimestamp());
        msrExamMapper.insert(msrExam);

        //科目分析
        Map<String, List<MsPaperStudent>> subjectCollect = paperStudents.stream().collect(Collectors.groupingBy(MsPaperStudent::getSubjectid));
        double fullscore = subjects.stream().mapToDouble(MsPaper::getPscore).sum();
        List<MsrSubject> msrSubjects = new ArrayList<>();
        MsrSubject msrSubject = new MsrSubject();
        msrSubject.setExamid(examid);
        msrSubject.setSubjectid("zf");
        msrSubject.setSubjectname("总分");
        msrSubject.setFullscore(fullscore);
        msrSubject.setMaxscore(schoolStatisticsMax);
        msrSubject.setMinscore(schoolStatisticsMin);
        msrSubject.setAvgscore(schoolStatisticsAverage);
        msrSubject.setScorerate(N_Utils.getDoubleDivideAndMulitiply(schoolStatisticsAverage, fullscore));
        msrSubject.setTotalnum(totalnum);
        msrSubject.setStunum(stunum);
        msrSubject.setMissnum(missnum);
        msrSubjects.add(msrSubject);
        for (MsPaper subject : subjects) {
            msrSubject = new MsrSubject();
            msrSubject.setExamid(examid);
            msrSubject.setSubjectid(subject.getSubjectid());
            msrSubject.setSubjectname(subject.getSubjectname());
            msrSubject.setFullscore(subject.getPscore());
            List<MsPaperStudent> studentList = subjectCollect.get(subject.getSubjectid());
            msrSubjects.add(msrSubject);
        }

        examMapper.updateExamState(examid, 3);
    }

    //分数排名
    private void setRank(List<MsrStudent> students, Map<String, Integer> rank, Integer code) {
        List<Map.Entry<Double, List<MsrStudent>>> gslist2 = students.stream()
                                                                    .collect(Collectors.groupingBy(MsrStudent::getStuscore)).entrySet()
                                                                    .stream().sorted((s1, s2) -> -Double.compare(s1.getKey(), s2.getKey())).collect(Collectors.toList());
        int index = 1;
        String key;
        for (Map.Entry<Double, List<MsrStudent>> entry : gslist2) {
            for (MsrStudent cgs : entry.getValue()) {
                Integer studentid = cgs.getStudentid();
                Integer classid = cgs.getClassid();
                String subjectid = cgs.getSubjectid();
                if (code == 1) {//学校学生
                    key = studentid.toString();
                } else if (code == 2) {//学校科目学生
                    key = subjectid + "_" + studentid;
                } else if (code == 3) {//班级学生
                    key = classid + "_" + studentid;
                } else {//班级科目学生
                    key = classid + "_" + subjectid + "_" + studentid;
                }
                rank.put(key, index);
            }
            index = index + entry.getValue().size();
        }
    }
}
